package com.project.common.mapper.cache;

import com.baomidou.mybatisplus.annotation.InterceptorIgnore;
import com.project.common.core.domain.entity.LargeCache;

import java.util.List;

@InterceptorIgnore(blockAttack = "true")
public interface PlatCacheMapper {
    LargeCache selectCache(String key);

    void insertCache(String key, String data);

    void deleteCache(String key);

    void deleteCacheObscure(String key);

    void deleteCaches(List<String> list);

    List<String> selectKeys();

    void clearAll();
}
